
import java.awt.*;

class JuliaPanel extends ZoomPanel {

	private double a, b;
    private int[] palette;

    int NITER;

    public JuliaPanel(double aa, double bb) {
        super(200, 200, -2, -2, 2, 2);
        a = aa; b = bb;
        palette = new int[256];
        for (int k = 0; k < 256; k++) {
            int k1 = 255 - (8 * k) % 256;
            int k2 = k1;
            int k3 = k1;
            palette[k] = (new Color(k1, k2, k3)).getRGB();
            palette[0] = (new Color(255, 0, 0)).getRGB();
        }
    }

    protected void resetBounds() {
        super.resetBounds();
        NITER = 64;
    }

    public void setCoords(double x, double y) {
        a = x; b = y;
        resetBounds();
        mustRecompute = true;
        repaint();
    }

    int iterer(double x, double y) {
        double u = x, v = y;
        int k = 0;
        while (k < NITER && u * u + v * v < 4) {
            double w = u * u - v * v + a;
            v = 2 * u * v + b;
            u = w;
            k++;
        }
        return k;
    }

    public void recompute() {
        double d = (xmax - xmin);
        NITER = Math.min(1024, (int) (64 * Math.max(1, 2 / Math.pow(d, 0.5))));
        for (int i = 0; i < NX; i++)
            for (int j = 0; j < NY; j++) {
                double x = getX(i, j);
                double y = getY(i, j);
                int k = iterer(x, y);
                if (k == NITER) k = 0;
                else if (k % 256 == 0) k = 1;
                else k = k % 256;
                img.setRGB(i, j, palette[k]);
            }
    }
}