
import java.awt.*;
import java.awt.event.*;

class MandelPanel extends ZoomPanel {

    private int NITER;
    protected double deltaFactor, iterFactor;
    protected long statIter;

    protected JuliaPanel jpan;
    protected OrbitPanel opan;

    protected MandelPanel(JuliaPanel jjpan, OrbitPanel oopan) {
        super(400, 400, -2, -1.5, 1, 1.5);
        jpan = jjpan;
        opan = oopan;
        statIter = 0;
    }

    protected void onMouseMoved(MouseEvent e) {
        int i = e.getX(), j = e.getY();
        double x = getX(i, j);
        double y = getY(i, j);
        if (!e.isShiftDown()) jpan.setCoords(x, y);
        if (!e.isControlDown()) opan.setCoords(x, y);
    }

    protected void resetBounds() {
        super.resetBounds();
        NITER = 256;
        deltaFactor = 1;
        iterFactor = 1;
    }

    private int iterer(double x, double y) {
        double delta = deltaFactor * (xmax - xmin) / NX / 8.;
        int niter = (int) (NITER * iterFactor);
        double ovf = 1e100;
        double u = 0, v = 0;
        int k = 0;
        while (k < niter && u * u + v * v < 4) {
            double w = u * u - v * v + x;
            v = 2 * u * v + y;
            u = w;
            k++;
            statIter++;
        }
        if (k == niter) return 0;
        else {
            u = 0; v = 0;
            double u1 = 0, v1 = 0;
            int j = 0;
            while (j <= k && u1 * u1 + v1 * v1 < ovf) {
                double w1 = 2 * (u * u1 - v * v1) + 1;
                v1 = 2 * (u * v1 + u1 * v);
                u1 = w1;
                double w = u * u - v * v + x;
                v = 2 * u * v + y;
                u = w;
                j++;
            }
            if (j <= k) return 0;
            else {
                double mzn = Math.sqrt(u * u + v * v);
                double mzn1 = Math.sqrt(u1 * u1 + v1 * v1);
                double dist = 2 * mzn * Math.log(mzn) / mzn1;
                if (dist <= delta) return 0;
                else return 255;
            }
        }
    }

    protected void recompute() {
        double d = (xmax - xmin) / 2;
        NITER = 256 * (int) (Math.max(1, Math.log(1.0 / d) / 2));
        final int white = (255 << 16) + (255 << 8) + 255;
        final int black = (0 << 16) + (0 << 8) + 0;

        for (int i = 0; i < NX; i++) {
            for (int j = 0; j < NY; j++) {
                double x = getX(i, j);
                double y = getY(i, j);
                int k = iterer(x, y);
                if (k == 0) img.setRGB(i, j, black);
                else img.setRGB(i, j, white);
            }
        }
        Graphics g = img.getGraphics();
        g.setColor(Color.red);
        g.drawRect(NX / 2 - 2, NY / 2 - 2, 5, 5);
    }
}
