
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

public class Mandelbrodt extends JApplet implements MouseListener, MouseMotionListener,
                                                    ActionListener, ChangeListener {

	private MandelPanel mpan;
    private JuliaPanel jpan;
    private OrbitPanel opan;

    private JTextField xText, yText, deltaText, iterText, statText;
    private JButton resetButton;
    private JSlider deltaSlider, iterSlider;

    public void init() {
        jpan = new JuliaPanel(0, 0);
        opan = new OrbitPanel(0, 0);
        mpan = new MandelPanel(jpan, opan);
        mpan.addMouseMotionListener(this);
        mpan.addMouseListener(this);
        add(mpan, "Center");
        add(juliaAndOrbitPanel(), "East");
        add(stuffPanel(), "South");
    }

    // Dessins Julia et orbite

    private JPanel juliaAndOrbitPanel() {
        JPanel p = new JPanel();
        p.setLayout(new BorderLayout(0, 0));
        p.add(jpan,"Center");
        p.add(opan, "South");
        return p;
    }

    //

    private JPanel stuffPanel() {
        JPanel p;
        p = new JPanel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
        p.add(xyPanel());
        p.add(slidersPanel());
        p.add(razAndStatPanel());
        return p;
    }

    // Champs x et y

    private JPanel xyPanel() {
        JPanel p1, px, py;
        p1 = new JPanel();
        p1.setLayout(new BorderLayout());

        px = new JPanel();
        px.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
        px.add(new JLabel("x"));
        xText = new JTextField(8);
        xText.setEditable(false);
        px.add(xText);

        py = new JPanel();
        py.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));

        py.add(new JLabel("y"));

        yText = new JTextField(8);
        yText.setEditable(false);
        py.add(yText);

        p1.add(px, "Center");
        p1.add(py, "South");
        return p1;
    }

    // Sliders delta et iter

    private JPanel slidersPanel() {
        JPanel p1, p2;
        p1 = new JPanel();
        p1.setLayout(new BorderLayout());

        p2 = new JPanel();
        p2.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));

        p2.add(new JLabel("delta"));
        deltaSlider = new JSlider(1, 50, 10);
        deltaSlider.addChangeListener(this);
        p2.add(deltaSlider);

        deltaText = new JTextField(5);
        deltaText.setEditable(false);
        deltaText.setText("1");
        p2.add(deltaText);

        p1.add(p2, "Center");

        p2 = new JPanel();
        p2.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
        p2.add(new JLabel("   iter"));
        iterSlider = new JSlider(1, 50, 10);
        iterSlider.addChangeListener(this);
        p2.add(iterSlider);

        iterText = new JTextField(5);
        iterText.setEditable(false);
        iterText.setText("1");
        p2.add(iterText);

        p1.add(p2, "South");
        return p1;
    }

    // Bouton R.A.Z et champ stats

    private JPanel razAndStatPanel() {
        JPanel p1, p2;
        p1 = new JPanel();
        p1.setLayout(new BorderLayout());

        resetButton = new JButton("R.A.Z.");
        resetButton.addActionListener(this);
        p1.add(resetButton, "Center");

        p2 = new JPanel();
        p2.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));

        p2.add(new JLabel("stat"));
        statText = new JTextField(10);
        statText.setEditable(false);
        statText.setText(mpan.statIter + "");
        p2.add(statText);

        p1.add(p2,"South");
        return p1;
    }

    // ActionListener

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == resetButton) {
            mpan.resetBounds();
            deltaSlider.setValue(10);
            deltaText.setText("1");
            iterSlider.setValue(10);
            iterText.setText("1");
            statText.setText(mpan.statIter + "");
            mpan.mustRecompute = true;
            mpan.repaint();
        }
    }

    // ChangeListener

    public void stateChanged(ChangeEvent e) {
        if (e.getSource() == deltaSlider) {
            double d = deltaSlider.getValue() / 10.;
            deltaText.setText(d + "");
            mpan.deltaFactor = d;
            statText.setText(mpan.statIter + "");
            if (!deltaSlider.getValueIsAdjusting()) {
                mpan.mustRecompute = true;
                mpan.repaint();
            }
        }
        else if (e.getSource() == iterSlider) {
            double d = iterSlider.getValue() / 10.;
            iterText.setText(d + "");
            mpan.iterFactor = d;
            statText.setText(mpan.statIter + "");
            if (!iterSlider.getValueIsAdjusting()) {
                mpan.mustRecompute = true;
                mpan.repaint();
            }
        }
    }

    // MouseListener

    public void mouseExited(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}

    public void mousePressed(MouseEvent e) {
        statText.setText(mpan.statIter + "");
    }

    public void mouseReleased(MouseEvent e) {}

    // MouseMotionListener

    public void mouseDragged(MouseEvent e) {}

    public void mouseMoved(MouseEvent e) {
        int i = e.getX();
        int j = e.getY();
        double d = 1e6;
        double x = mpan.getX(i, j);
        x = (int) (d * x) / d;
        xText.setText(x + "");

        double y = mpan.getY(i, j);
        y = (int) (d * y) / d;
        yText.setText(y + "");
        statText.setText(mpan.statIter + "");
    }

    // Main

    public static void main(String[] args) {
        JFrame f = new JFrame("Ensemble de Mandelbrot");
        Mandelbrodt tf = new Mandelbrodt();
        tf.init();
        f.add(tf);
        f.pack();
        f.setResizable(false);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setVisible(true);
    }
}
