import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

class ZoomPanel extends JPanel {

    protected int NX, NY;

    private double xmin0, xmax0, ymin0, ymax0;
    protected double xmin, xmax, ymin, ymax;

    protected BufferedImage img;

    protected boolean mustRecompute;

    protected ZoomPanel(int NX, int NY, double x1, double y1, double x2, double y2) {
        this.NX = NX;
        this.NY = NY;
        xmin0 = x1;
        ymin0 = y1;
        xmax0 = x2;
        ymax0 = y2;
        resetBounds();
        mustRecompute = true;

        img = new BufferedImage(NX, NY, BufferedImage.TYPE_INT_RGB);

        ZoomPanelAdapter zpa = new ZoomPanelAdapter(this);
        addMouseMotionListener(zpa);
        addMouseListener(zpa);
    }

    protected void resetBounds() {
        xmin = xmin0;
        xmax = xmax0;
        ymin = ymin0;
        ymax = ymax0;
    }

    protected double getX(int i, int j) {
        return xmin + i * (xmax-xmin) / NX;
    }

    protected double getY(int i, int j) {
        return ymax + j * (ymin - ymax) / NY;
    }

    protected void zoom(int i, int j, double k) {
        double xc = getX(i, j);
        double yc = getY(i, j);
        double d = (xmax - xmin) / 2;
        double d1 = k * d;
        xmin = xc - d1;
        xmax = xc + d1;
        ymin = yc - d1;
        ymax = yc + d1;
        mustRecompute = true;
        repaint();
    }

    protected void recompute() {}

     public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (mustRecompute) {
            recompute();
            mustRecompute = false;
        }
        g.drawImage(img, 0, 0, null);
    }

    protected void onMouseMoved(MouseEvent e) {}

    public Dimension getPreferredSize() { return new Dimension(NX, NY); }
}
