
import java.awt.*;
import java.awt.event.*;

class ZoomPanelAdapter implements MouseListener, MouseMotionListener {

    private ZoomPanel zp;
    private int xSelect, ySelect;

    protected ZoomPanelAdapter(ZoomPanel zp) {
        this.zp = zp;
    }

    private boolean isInsideScreen(int x, int y, int d) {
        int x1 = x - d;
        int y1 = y - d;
        int x2 = x + d;
        int y2 = y + d;
        return (x1 >= 0 && x1 < zp.NX-1 && y1 >= 0 && y1 < zp.NY-1 &&
                x2 >= 0 && x2 < zp.NX-1 && y2 >= 0 && y2 < zp.NY-1);
    }

    private int distance(int x1, int y1, int x2, int y2) {
        return Math.max(Math.abs(x2 - x1), Math.abs(y2 - y1));
    }

    public void mouseDragged(MouseEvent e) {
        Graphics g = zp.getGraphics();
        g.drawImage(zp.img, 0, 0, null);
        int x = e.getX();
        int y = e.getY();
        int d = distance(xSelect, ySelect, x, y);
        int x1 = xSelect - d;
        int y1 = ySelect - d;
        int w = 2 * d + 1;
        if (isInsideScreen(xSelect, ySelect, d)) {
            g.setColor(Color.blue);
            g.drawRect(xSelect - d, ySelect - d, w, w);
        }
    }

    public void mouseMoved(MouseEvent e) {
        zp.onMouseMoved(e);
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void mousePressed(MouseEvent e) {
        xSelect = e.getX();
        ySelect = e.getY();
    }

    public void mouseReleased(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        if (x >= 0 && x < zp.NX && y >= 0 && y < zp.NY)
            if (distance(xSelect, ySelect, x, y) < 5)
                if (e.getButton() == MouseEvent.BUTTON1)
                    zp.zoom(x, y, 1);
                else
                    zp.zoom(x, y, 2);
            else {
                int d = distance(xSelect, ySelect, x, y);
                if (isInsideScreen(xSelect, ySelect, d))
                    zp.zoom(xSelect, ySelect, 2. * d / zp.NX);
            }
    }
}